@@ -107,11 +107,18 @@ class Directories:
 
         :returns: Directories object
         """
+        # only set CONTAINER_VAR_LIBS_FOLDER/CONTAINER_CACHE_FOLDER inside the container to redirect var_libs/cache to
+        # another directory to avoid override by host mount
+        var_libs = (
+            os.environ.get("CONTAINER_VAR_LIBS_FOLDER", "").strip()
+            or "/var/lib/localstack/var_libs"
+        )
+        cache = os.environ.get("CONTAINER_CACHE_FOLDER", "").strip() or "/var/lib/localstack/cache"
         return Directories(
             static_libs=INSTALL_DIR_INFRA,
-            var_libs="/var/lib/localstack/var_libs",
-            cache="/var/lib/localstack/cache",
-            tmp=TMP_FOLDER,  # TODO: move to /var/lib/localstack/tmp
+            var_libs=var_libs,
+            cache=cache,
+            tmp=TMP_FOLDER,  # TODO: move to /var/lib/localstack/tmp - or /tmp/localstack
             functions=HOST_TMP_FOLDER,  # TODO: move to /var/lib/localstack/tmp
             data=DATA_DIR,  # TODO: move to /var/lib/localstack/data
             config=None,  # config directory is host-only
@@ -358,6 +365,9 @@ EAGER_SERVICE_LOADING = is_env_true("EAGER_SERVICE_LOADING")
 # Whether to skip downloading additional infrastructure components (e.g., custom Elasticsearch versions)
 SKIP_INFRA_DOWNLOADS = os.environ.get("SKIP_INFRA_DOWNLOADS", "").strip()
 
+# whether to enable legacy record&replay persistence mechanism (default true, but will be disabled in a future release!)
+LEGACY_PERSISTENCE = is_env_not_false("LEGACY_PERSISTENCE")
+
 # Adding Stepfunctions default port
 LOCAL_PORT_STEPFUNCTIONS = int(os.environ.get("LOCAL_PORT_STEPFUNCTIONS") or 8083)
 # Stepfunctions lambda endpoint override
@@ -417,7 +427,7 @@ LAMBDA_FALLBACK_URL = os.environ.get("LAMBDA_FALLBACK_URL", "").strip()
 # endpoint (can use useful for advanced test setups)
 LAMBDA_FORWARD_URL = os.environ.get("LAMBDA_FORWARD_URL", "").strip()
 # Time in seconds to wait at max while extracting Lambda code.
-# By default it is 25 seconds for limiting the execution time
+# By default, it is 25 seconds for limiting the execution time
 # to avoid client/network timeout issues
 LAMBDA_CODE_EXTRACT_TIME = int(os.environ.get("LAMBDA_CODE_EXTRACT_TIME") or 25)
 
@@ -426,6 +436,10 @@ LAMBDA_CODE_EXTRACT_TIME = int(os.environ.get("LAMBDA_CODE_EXTRACT_TIME") or 25)
 # For example: "my-first-stream:1,my-other-stream:2,my-last-stream:1"
 KINESIS_INITIALIZE_STREAMS = os.environ.get("KINESIS_INITIALIZE_STREAMS", "").strip()
 
+# URL to a custom elasticsearch backend cluster. If this is set to a valid URL, then localstack will not create
+# elasticsearch cluster instances, but instead forward all domains to the given backend.
+ES_CUSTOM_BACKEND = os.environ.get("ES_CUSTOM_BACKEND", "").strip()
+
 # Strategy used when creating elasticsearch domain endpoints routed through the edge proxy
 # valid values: domain | path | off
 ES_ENDPOINT_STRATEGY = os.environ.get("ES_ENDPOINT_STRATEGY", "").strip() or "domain"
@@ -459,6 +473,7 @@ CONFIG_ENV_VARS = [
     "DYNAMODB_ERROR_PROBABILITY",
     "DYNAMODB_READ_ERROR_PROBABILITY",
     "DYNAMODB_WRITE_ERROR_PROBABILITY",
+    "ES_CUSTOM_BACKEND",
     "ES_ENDPOINT_STRATEGY",
     "ES_MULTI_CLUSTER",
     "DOCKER_BRIDGE_IP",
@@ -512,6 +527,7 @@ CONFIG_ENV_VARS = [
     "REQUESTS_CA_BUNDLE",
     "LEGACY_DOCKER_CLIENT",
     "EAGER_SERVICE_LOADING",
+    "LAMBDA_STAY_OPEN_MODE",
 ]
 
 for key, value in six.iteritems(DEFAULT_SERVICE_PORTS):
@@ -590,9 +606,11 @@ def in_docker():
     except Exception:
         pass
     with open("/proc/1/cgroup", "rt") as ifh:
-        os_hostname = open("/etc/hostname", "rt").read().strip()
         content = ifh.read()
-        if os_hostname in content or "docker" in content:
+        if "docker" in content:
+            return True
+        os_hostname = socket.gethostname()
+        if os_hostname and os_hostname in content:
             return True
     return False
 
@@ -630,6 +648,9 @@ except socket.error:
 if is_in_docker and not os.environ.get("LAMBDA_REMOTE_DOCKER", "").strip():
     LAMBDA_REMOTE_DOCKER = True
 
+# whether lambdas should use stay open mode if executed in "docker-reuse" executor
+LAMBDA_STAY_OPEN_MODE = is_in_docker and is_env_not_false("LAMBDA_STAY_OPEN_MODE")
+
 # set variables no_proxy, i.e., run internal service calls directly
 no_proxy = ",".join(set((LOCALSTACK_HOSTNAME, LOCALHOST, LOCALHOST_IP, "[::1]")))
 if os.environ.get("no_proxy"):
