@@ -9,7 +9,7 @@ LOG = logging.getLogger(__name__)
 @hooks.configure_localstack_container()
 def configure_edge_port(container):
     ports = [config.EDGE_PORT, config.EDGE_PORT_HTTP]
-    LOG.info("configuring container with edge ports: %s", ports)
+    LOG.debug("configuring container with edge ports: %s", ports)
     for port in ports:
         if port:
             container.ports.add(port)
