@@ -56,6 +56,7 @@ from localstack.utils.common import (
     timestamp,
     to_bytes,
     to_str,
+    truncate,
     wait_for_port_open,
 )
 from localstack.utils.docker_utils import (
@@ -177,7 +178,7 @@ class InvocationContext:
     event: LambdaEvent
     lambda_command: Union[str, List[str]]  # TODO: change to List[str] ?
     docker_flags: Union[str, List[str]]  # TODO: change to List[str] ?
-    environment: Dict[str, str]
+    environment: Dict[str, Optional[str]]
     context: LambdaContext
     invocation_type: str  # "Event" or "RequestResponse"
 
@@ -637,11 +638,8 @@ class LambdaExecutorContainers(LambdaExecutor):
                 additional_logs = "\n".join(lines[:idx] + lines[idx + 1 :])
                 log_output += "\n%s" % additional_logs
 
-        log_formatted = log_output.strip().replace("\n", "\n> ")
         func_arn = lambda_function and lambda_function.arn()
-        LOG.debug(
-            "Lambda %s result / log output:\n%s\n> %s" % (func_arn, result.strip(), log_formatted)
-        )
+        log_lambda_result(func_arn, result, log_output)
 
         # store log output - TODO get live logs from `process` above?
         store_lambda_logs(lambda_function, log_output)
@@ -804,6 +802,8 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
         if not self._should_use_stay_open_mode(lambda_docker_ip, check_port=True):
             LOG.debug("Using 'docker exec' to run invocation in docker-reuse Lambda container")
+            # disable stay open mode for this one, to prevent starting runtime API server
+            env_vars["DOCKER_LAMBDA_STAY_OPEN"] = None
             return DOCKER_CLIENT.exec_in_container(
                 container_name_or_id=container_info.name,
                 command=inv_context.lambda_command,
@@ -852,9 +852,9 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
     ) -> bool:
         """Return whether to use stay-open execution mode - if we're running in Docker, the given IP
         is defined, and if the target API endpoint is available (optionally, if check_port is True)."""
-        should_use = lambda_docker_ip and in_docker()
+        should_use = lambda_docker_ip and config.LAMBDA_STAY_OPEN_MODE
         if not should_use or not check_port:
-            return False
+            return should_use
         full_url = self._get_lambda_stay_open_url(lambda_docker_ip)
         return is_port_open(full_url)
 
@@ -983,8 +983,10 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         entrypoint = "/bin/bash"
         interactive = True
 
-        if in_docker():
+        if config.LAMBDA_STAY_OPEN_MODE:
             env_vars["DOCKER_LAMBDA_STAY_OPEN"] = "1"
+            # clear docker lambda use stdin since not relevant with stay open
+            env_vars.pop("DOCKER_LAMBDA_USE_STDIN", None)
             entrypoint = None
             command = [lambda_function.handler]
             interactive = False
@@ -1252,11 +1254,8 @@ class LambdaExecutorLocal(LambdaExecutor):
                 additional_logs = "\n".join(lines[:idx] + lines[idx + 1 :])
                 log_output += "\n%s" % additional_logs
 
-        log_formatted = log_output.strip().replace("\n", "\n> ")
         func_arn = lambda_function and lambda_function.arn()
-        LOG.debug(
-            "Lambda %s result / log output:\n%s\n> %s" % (func_arn, result.strip(), log_formatted)
-        )
+        log_lambda_result(func_arn, result, log_output)
 
         # store log output - TODO get live logs from `process` above?
         # store_lambda_logs(lambda_function, log_output)
@@ -1554,6 +1553,15 @@ class Util:
         return env_vars
 
 
+def log_lambda_result(func_arn, result, log_output):
+    result = to_str(result or "")
+    log_output = truncate(to_str(log_output or ""), max_length=1000)
+    log_formatted = truncate(log_output.strip().replace("\n", "\n> "), max_length=1000)
+    LOG.debug(
+        "Lambda %s result / log output:\n%s\n> %s" % (func_arn, result.strip(), log_formatted)
+    )
+
+
 # --------------
 # GLOBAL STATE
 # --------------
