@@ -79,14 +79,7 @@ SFN_PATCH_CLASS1 = "com/amazonaws/stepfunctions/local/runtime/Config.class"
 SFN_PATCH_CLASS2 = (
     "com/amazonaws/stepfunctions/local/runtime/executors/task/LambdaTaskStateExecutor.class"
 )
-SFN_PATCH_CLASS_URL1 = "%s/raw/master/stepfunctions-local-patch/%s" % (
-    ARTIFACTS_REPO,
-    SFN_PATCH_CLASS1,
-)
-SFN_PATCH_CLASS_URL2 = "%s/raw/master/stepfunctions-local-patch/%s" % (
-    ARTIFACTS_REPO,
-    SFN_PATCH_CLASS2,
-)
+SFN_PATCH_URL_PREFIX = f"{ARTIFACTS_REPO}/raw/master/stepfunctions-local-patch"
 
 # kinesis-mock version
 KINESIS_MOCK_VERSION = os.environ.get("KINESIS_MOCK_VERSION") or "0.2.0"
@@ -107,7 +100,7 @@ SQS_BACKEND_IMPL = os.environ.get("SQS_PROVIDER") or "moto"
 # GO Lambda runtime
 GO_RUNTIME_VERSION = "0.4.0"
 GO_RUNTIME_DOWNLOAD_URL_TEMPLATE = "https://github.com/localstack/awslamba-go-runtime/releases/download/v{version}/awslamba-go-runtime-{version}-{os}-{arch}.tar.gz"
-GO_INSTALL_FOLDER = os.path.join(config.dirs.tmp, "awslamba-go-runtime")
+GO_INSTALL_FOLDER = os.path.join(config.dirs.var_libs, "awslamba-go-runtime")
 GO_LAMBDA_RUNTIME = os.path.join(GO_INSTALL_FOLDER, "aws-lambda-mock")
 GO_LAMBDA_MOCKSERVER = os.path.join(GO_INSTALL_FOLDER, "mockserver")
 
@@ -118,6 +111,13 @@ TERRAFORM_URL_TEMPLATE = (
 )
 TERRAFORM_BIN = os.path.join(dirs.static_libs, f"terraform-{TERRAFORM_VERSION}", "terraform")
 
+# Java Test Jar Download (used for tests)
+TEST_LAMBDA_JAVA = os.path.join(config.dirs.var_libs, "localstack-utils-tests.jar")
+MAVEN_BASE_URL = "https://repo.maven.apache.org/maven2"
+TEST_LAMBDA_JAR_URL = "{url}/cloud/localstack/{name}/{version}/{name}-{version}-tests.jar".format(
+    version=LOCALSTACK_MAVEN_VERSION, url=MAVEN_BASE_URL, name="localstack-utils"
+)
+
 
 def get_elasticsearch_install_version(version: str) -> str:
     from localstack.services.es import versions
@@ -338,19 +338,17 @@ def install_stepfunctions_local():
             file.rename(Path(INSTALL_DIR_STEPFUNCTIONS) / file.name)
         rm_rf("%s/stepfunctionslocal" % dirs.static_libs)
     # apply patches
-    for patch_class, patch_url in (
-        (SFN_PATCH_CLASS1, SFN_PATCH_CLASS_URL1),
-        (SFN_PATCH_CLASS2, SFN_PATCH_CLASS_URL2),
-    ):
-        patch_class_file = os.path.join(INSTALL_DIR_STEPFUNCTIONS, patch_class)
-        if not os.path.exists(patch_class_file):
-            download(patch_url, patch_class_file)
-            cmd = 'cd "%s"; zip %s %s' % (
-                INSTALL_DIR_STEPFUNCTIONS,
-                INSTALL_PATH_STEPFUNCTIONS_JAR,
-                patch_class,
-            )
-            run(cmd)
+    for patch_class in (SFN_PATCH_CLASS1, SFN_PATCH_CLASS2):
+        patch_url = f"{SFN_PATCH_URL_PREFIX}/{patch_class}"
+        add_file_to_jar(patch_class, patch_url, target_jar=INSTALL_PATH_STEPFUNCTIONS_JAR)
+
+
+def add_file_to_jar(class_file, class_url, target_jar, base_dir=None):
+    base_dir = base_dir or os.path.dirname(target_jar)
+    patch_class_file = os.path.join(base_dir, class_file)
+    if not os.path.exists(patch_class_file):
+        download(class_url, patch_class_file)
+        run(["zip", target_jar, class_file], cwd=base_dir)
 
 
 def install_dynamodb_local():
@@ -407,6 +405,13 @@ def install_lambda_java_libs():
         download(URL_LOCALSTACK_FAT_JAR, INSTALL_PATH_LOCALSTACK_FAT_JAR)
 
 
+def install_lambda_java_testlibs():
+    # Download the LocalStack Utils Test jar file from the maven repo
+    if not os.path.exists(TEST_LAMBDA_JAVA):
+        mkdir(os.path.dirname(TEST_LAMBDA_JAVA))
+        download(TEST_LAMBDA_JAR_URL, TEST_LAMBDA_JAVA)
+
+
 def install_go_lambda_runtime():
     if os.path.isfile(GO_LAMBDA_RUNTIME):
         return
@@ -479,9 +484,9 @@ def install_components(names):
 
 
 def install_all_components():
-    hooks.install.run()
-    # install all components
+    # install dependencies - make sure that install_components(..) is called before hooks.install below!
     install_components(DEFAULT_SERVICE_PORTS.keys())
+    hooks.install.run()
 
 
 def install_debugpy_and_dependencies():
@@ -547,6 +552,7 @@ def download_and_extract_with_retry(archive_url, tmp_archive, target_dir):
         download_and_extract(archive_url, target_dir, tmp_archive=tmp_archive)
 
 
+# kept here for backwards compatibility (installed on "make init" - TODO should be removed)
 installers = {
     "cloudformation": install_cloudformation_libs,
     "dynamodb": install_dynamodb_local,
@@ -612,7 +618,7 @@ class InstallerManager:
 
 def main():
     if len(sys.argv) > 1:
-        # set API key so pro install hooks are called
+        # set test API key so pro install hooks are called
         os.environ["LOCALSTACK_API_KEY"] = os.environ.get("LOCALSTACK_API_KEY") or "test"
         if sys.argv[1] == "libs":
             print("Initializing installation.")
@@ -622,6 +628,7 @@ def main():
         if sys.argv[1] in ("libs", "testlibs"):
             # Install additional libraries for testing
             install_amazon_kinesis_client_libs()
+            install_lambda_java_testlibs()
         print("Done.")
 
 
