@@ -3,6 +3,7 @@ import logging
 import os
 import re
 import shlex
+import signal
 import threading
 import warnings
 from functools import wraps
@@ -670,6 +671,25 @@ def start_infra_in_docker():
     log_printer = FileListener(container.logfile, print)
     log_printer.start()
 
+    # Set up signal handler, to enable clean shutdown across different operating systems.
+    #  There are subtle differences across operating systems and terminal emulators when it
+    #  comes to handling of CTRL-C - in particular, Linux sends SIGINT to the parent process,
+    #  whereas MacOS sends SIGINT to the process group, which can result in multiple SIGINT signals
+    #  being received (e.g., when running the localstack CLI as part of an "npm run .." script).
+    #  Hence, using a shutdown handler and synchronization event here, to avoid inconsistencies.
+    def shutdown_handler(*args):
+        with shutdown_event_lock:
+            if shutdown_event.is_set():
+                return
+            shutdown_event.set()
+        print("Shutting down...")
+        server.shutdown()
+        log_printer.close()
+
+    shutdown_event = threading.Event()
+    shutdown_event_lock = threading.RLock()
+    signal.signal(signal.SIGINT, shutdown_handler)
+
     # start the Localstack container as a Server
     server = LocalstackContainerServer(container)
     try:
@@ -677,9 +697,7 @@ def start_infra_in_docker():
         server.join()
     except KeyboardInterrupt:
         print("ok, bye!")
-    finally:
-        server.shutdown()
-        log_printer.close()
+        shutdown_handler()
 
 
 def start_infra_in_docker_detached(console):
